@@ -183,7 +183,7 @@ g_input_stream_read  (GInputStream  *stream,
   if (((gssize) count) < 0)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
-		   _("Too large count value passed to %s"), __FUNCTION__);
+		   _("Too large count value passed to %s"), G_STRFUNC);
       return -1;
     }
 
@@ -315,7 +315,7 @@ g_input_stream_skip (GInputStream  *stream,
   if (((gssize) count) < 0)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
-		   _("Too large count value passed to %s"), __FUNCTION__);
+		   _("Too large count value passed to %s"), G_STRFUNC);
       return -1;
     }
   
@@ -557,7 +557,7 @@ g_input_stream_read_async (GInputStream        *stream,
 					   user_data,
 					   G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
 					   _("Too large count value passed to %s"),
-					   __FUNCTION__);
+					   G_STRFUNC);
       return;
     }
 
@@ -682,7 +682,7 @@ g_input_stream_skip_async (GInputStream        *stream,
 					   user_data,
 					   G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
 					   _("Too large count value passed to %s"),
-					   __FUNCTION__);
+					   G_STRFUNC);
       return;
     }
 
