@@ -15,6 +15,7 @@ g_app_info_create_flags_get_type (void)
       static const GFlagsValue values[] = {
         { G_APP_INFO_CREATE_NONE, "G_APP_INFO_CREATE_NONE", "none" },
         { G_APP_INFO_CREATE_NEEDS_TERMINAL, "G_APP_INFO_CREATE_NEEDS_TERMINAL", "needs-terminal" },
+        { G_APP_INFO_CREATE_SUPPORTS_URIS, "G_APP_INFO_CREATE_SUPPORTS_URIS", "supports-uris" },
         { 0, NULL, NULL }
       };
       GType g_define_type_id =
@@ -110,6 +111,25 @@ g_file_create_flags_get_type (void)
   return g_define_type_id__volatile;
 }
 
+GType
+g_mount_mount_flags_get_type (void)
+{
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_define_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { G_MOUNT_MOUNT_NONE, "G_MOUNT_MOUNT_NONE", "none" },
+        { 0, NULL, NULL }
+      };
+      GType g_define_type_id =
+        g_enum_register_static (g_intern_static_string ("GMountMountFlags"), values);
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+    }
+
+  return g_define_type_id__volatile;
+}
+
 GType
 g_mount_unmount_flags_get_type (void)
 {
